import 'package:built_redux/built_redux.dart';

CReducer<S, B, P> decorateReducer<S, B, P>(
  CReducer<S, B, P> decoratedReducer,
  bool Function(S state, Action<P> payload) condition, {
  bool throwException = false,
}) {
  assert(decoratedReducer != null);
  assert(condition != null);

  return (state, action, builder) {
    if (!condition(state, action)) {
      assert(!throwException, 'Condition failed for ${action.name}.');
      return;
    }

    decoratedReducer(state, action, builder);
  };
}

CReducer<S, B, Iterable<I>> ensureNotEmpty<S, B, I>(
  CReducer<S, B, Iterable<I>> decoratedReducer, {
  bool throwException,
}) {
  return decorateReducer(
    decoratedReducer,
    (state, action) => action.payload.isNotEmpty,
    throwException: throwException ?? false,
  );
}

CReducer<S, B, Iterable<I>> ensureNotNullNorEmptyPayload<S, B, I>(
  CReducer<S, B, Iterable<I>> decoratedReducer, {
  bool throwException,
}) {
  return ensureNotNullPayload(
    ensureNotEmpty(
      decoratedReducer,
      throwException: throwException,
    ),
    throwException: throwException,
  );
}

CReducer<S, B, P> ensureNotNullPayload<S, B, P>(
  CReducer<S, B, P> decoratedReducer, {
  bool throwException,
}) {
  return decorateReducer(
      decoratedReducer, (state, payload) => payload.payload != null,
      throwException: throwException ?? false);
}
